/* 20 Dec 2005

   Interface from FORTRAN/GAMESS to C/system calls
   ===============================================
 
   The amount of quiche is kept to a minimum, as there are only 4 routines!
          The first three routines amount to a FORTRAN wrapper around
          the C calls malloc and free, by Steve Elbert in January 1990.
   memory allocation: function invocation only: LOCMEM = MEMGET(NWORDS)
          NWORDS is the no. of 8-byte words of memory requested.
          The return value is the address of the allocated memory, if
          this is zero, the system doesn't have the memory available.
   memory release: subroutine call only: CALL MEMREL(LOCMEM)
          Deallocate all memory above the address LOCMEM.
          (deallocated memory is not returned to system)
   memory address: function invocation only: LOCX = LADDRS(X)
          Returns the absolute address of X, in bytes.  This is needed
          only if the FORTRAN library does not include the LOC function.
   process sleeping: subroutine call only: CALL NAPTIME(seconds)
          which is used when MPI is being used to pass messages, and
          allows DDI data servers to sleep when they are not in use.

   Please note: 
   There really shouldn't be anything here for a machine other than these 4,
   so one should refrain from adding anything else to this file at all.
   Other routines are present only because of inadequate system libraries.
*/

/*--------------- Compaq (aka Digital) --------------*/

#ifdef COMPAQ
#include <stdlib.h>

long memget_(nwords) long *nwords;
   { size_t nbytes;
     nbytes = (size_t) (*nwords+2)*8;
     return (long) malloc(nbytes); }

#include <unistd.h>
void naptime_(nptime) int *nptime;
   { unsigned int istat;
     istat=sleep(*nptime); }

#endif
 
/*--------------------------- Cray --------------------------*/
/*  --  Cray PVP or T3E --   */

#ifdef CRAY

int MEMGET(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void MEMREL(locmem) int *locmem;
   { free(*locmem); }

#include <unistd.h>
void NAPTIME(nptime) int *nptime;
   { unsigned int istat;
     istat=sleep(*nptime); }

#endif

/*  --  Cray X1 --   */

#ifdef CRAYX1

#include <unistd.h>

long memget_(nwords) long *nwords;
   { size_t nbytes;
     nbytes = (size_t) (*nwords+2)*8;
     return (long) malloc(nbytes); }

void memrel_(locmem) long *locmem;
   { free(*locmem); }

void naptime_(nptime) int *nptime;
   { unsigned int istat;
     istat=sleep(*nptime); }

#endif

/*  --  Cray XD1 --   */

#ifdef CRAYXD1
   
#include <stdlib.h>
#include <malloc.h>
#define FORTINT long

FORTINT memget_(nwords) FORTINT *nwords;
   { size_t nbytes;
     nbytes = (*nwords+2)*8;
     return (FORTINT) malloc(nbytes); }

void memrel_(locmem) FORTINT *locmem;
   { free((void*)*locmem); }

#include <unistd.h>
void naptime_(nptime) FORTINT *nptime;
   { sleep((unsigned int) *nptime); }

#endif


/*  --  Cray XT3 --   */

#ifdef CRAYXT3

#include <stdlib.h>
#include <malloc.h>
#define FORTINT long

FORTINT memget_(nwords) FORTINT *nwords;
   { size_t nbytes;
     nbytes = (*nwords+2)*8;
     return (FORTINT) malloc(nbytes); }

void memrel_(locmem) FORTINT *locmem;
   { free((void*)*locmem); }

#include <unistd.h>
void naptime_(nptime) FORTINT *nptime;
   { sleep((unsigned int) *nptime); }

#endif



/*-------------- Fujitsu VPP and AP ---------------*/

#ifdef FUJITSU

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

int laddrs_(arg) int arg;
   { return(arg); }

/*   This is the only machine type missing a NAPTIME implementation   */
/*   If you are trying to link GAMESS, please send a copy of your     */
/*   version of NAPTIME (many examples here!) to Mike Schmidt         */

#endif

/*--------------------- Hewlett-Packard --------------------------*/
/*    this version uses FORTRAN callable library routines only    */
/*                 no need to compile this file                   */



/*----------------- International Business Machines ----------------*/
/*  IBM32    = AIX operating system, 32 bits                        */
/*  IBM64    = AIX operating system, 64 bits, includes SP machines  */
/*  IBMBG    = Linux operating system, 32 bits, used on Blue Gene.  */
/*  IBMPPC64 = Linux operating system, 64 bits, e.g. OpenPower      */
 
#ifdef IBM32

int memget(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel(locmem) int *locmem;
   { free(*locmem); }

int laddrs(arg) int arg;
   { return(arg); }

#include <unistd.h>
void naptime(nptime) useconds_t *nptime;
   { int istat;
     istat=sleep(*nptime); }

#endif

#ifdef IBM64

#include <stdlib.h>
long memget(long *nwords)
  { size_t nbytes;
    nbytes = (*nwords + 2L) * 8L;
    return (long) malloc (nbytes); }

void memrel(long **locmem)
  { free (*locmem); }

long laddrs(void *arg)
  { return (long) arg; }

#include <unistd.h>
void naptime(long *nptime)
  { int istat;
    useconds_t delay;
    delay = (useconds_t) *nptime;
    istat=sleep(delay); }

#endif

#ifdef IBMBG

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/times.h>

int memget(int *nwords)
{
   int nbytes;
   nbytes = (*nwords+2)*8;
  return (int)malloc(nbytes);
}

void memrel(int *locmem)
{
   free((void *)*locmem);
}

int laddrs(arg) int arg;
   { return(arg); }

void naptime(nptime) int *nptime;
   { unsigned int istat;
     istat=sleep(*nptime); }

/*
     etime - return elapsed execution time
     double precision function etime (tarray)
     real tarray(2)
     written by Klaus-Peter Gulden, January 3, 1994
     note: CLK_TCK counts hundredths of seconds
*/

double etime(float *a)
   { double        elapsed;
     struct tms    buf;
     elapsed= (float) times(&buf)/ (float) CLK_TCK;
     a[0]=(float)(buf.tms_utime + buf.tms_cutime)/CLK_TCK;
     a[1]=(float)(buf.tms_stime + buf.tms_cstime)/CLK_TCK;
     return(elapsed); }

/*
     fdate - return date and time in an ASCII string
     subroutine fdate(string)
     character*24 string

     returns the current date and time as a 24 character string in the
     format of ctime(3).  Neither `newline' nor NULL will be included.
     by Istvan Cserny, June 21, 1995
*/

void fdate(char *cht, int cht_len)
   { static time_t t;
     time(&t);
     strncpy(cht,ctime(&t),24L);
     return; }

#endif

#ifdef IBMPPC64

#include <stdlib.h>
long memget(long *nwords)
  { size_t nbytes;
    nbytes = (*nwords + 2L) * 8L;
    return (long) malloc (nbytes); }

void memrel(long **locmem)
  { free (*locmem); }

long laddrs(void *arg)
  { return (long) arg; }

#include <unistd.h>
void naptime(long *nptime)
  { int istat;
    int delay;
    delay = (int) *nptime;
    istat=sleep(delay); }

/*
 *      etime - return elapsed execution time
 *      double precision function etime (tarray)
 *      real tarray(2)
 *      written by Klaus-Peter Gulden, January 3, 1994
 *      note: CLK_TCK counts hundredths of seconds
 */
#include <time.h>
#include <sys/times.h>

double etime(float *a)
   { double        elapsed;
     struct tms    buf;
     elapsed= (float) times(&buf)/ (float) CLK_TCK;
     a[0]=(float)(buf.tms_utime + buf.tms_cutime)/CLK_TCK;
     a[1]=(float)(buf.tms_stime + buf.tms_cstime)/CLK_TCK;
     return(elapsed); }

#endif

/*----- 32 bit PC (e.g. Pentium, Athlon, ...) running Linux -------*\
\*------- this also includes the Apple G4 running MAC OS X --------*/

#ifdef LINUX

int memget_(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) int *locmem;
   { free(*locmem); }

int laddrs_(arg) int arg;
   { return(arg); }

#include <unistd.h>
void naptime_(nptime) int *nptime;
   { unsigned int istat;
     istat=sleep(*nptime); }

/*
     etime - return elapsed execution time
     double precision function etime (tarray)
     real tarray(2)
     written by Klaus-Peter Gulden, January 3, 1994
     note: CLK_TCK counts hundredths of seconds, usually, but is
           defined by the system, through time.h's includes.
*/
/*
#include <time.h>
#include <sys/times.h>

double xetime_(float *a)
   { double        elapsed;
     struct tms    buf;
     elapsed= (float) times(&buf)/CLK_TCK;
     a[0]= (float) (buf.tms_utime + buf.tms_cutime)/CLK_TCK;
     a[1]= (float) (buf.tms_stime + buf.tms_cstime)/CLK_TCK;
     return(elapsed); }
*/

/*
     fdate - return date and time in an ASCII string
     subroutine fdate(string)
     character*24 string

     returns the current date and time as a 24 character string in the
     format of ctime(3).  Neither `newline' nor NULL will be included.
     by Istvan Cserny, June 21, 1995
*/

/* void fdate_(char *cht, int cht_len) */
/* { */
/*     static time_t t; */
/*     time(&t); */
/*     strncpy(cht,ctime(&t),24L); */
/*     return; */
/* } */
/* needed by pgi */
/* void f__xargc(){ return 0; } */

#endif

/*--- Absoft compiler...one might find this in Linux or MAC OS X worlds ---*/

#ifdef ABSOFT

int MEMGET(nwords) int *nwords;
   { int nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void MEMREL(locmem) int *locmem;
   { free(*locmem); }

int LADDRS(arg) int arg;
   { return(arg); }

#include <unistd.h>
void NAPTIME(nptime) int *nptime;
   { unsigned int istat;
     istat=sleep(*nptime); }

#endif

/*----------------- Itanium (IA64) running Linux -----------------*/
/* This is also good for AMD64  */

#ifdef LINUXIA64

#include <stdlib.h>
#include <malloc.h>
#define FORTINT long

FORTINT memget_(nwords) FORTINT *nwords;
   { size_t nbytes;
     nbytes = (*nwords+2)*8;
     return (FORTINT) malloc(nbytes); }

void memrel_(locmem) FORTINT *locmem;
   { free((void*)*locmem); }

#include <unistd.h>
void naptime_(nptime) FORTINT *nptime;
   { sleep((unsigned int) *nptime); }

#endif
 
/*----------------- NEC SX series -----------------*/

#ifdef NECSX
#include <unistd.h>

void memget_(nwords,sxaddr)
 long long *nwords, **sxaddr ;
   {char *malloc(),*card;
    unsigned nbytes;
    nbytes = (unsigned) ((*nwords+2)*8);
    card =  malloc (nbytes);
    *sxaddr = (long long *) card; }

void memrel_(locmem)
 long long *locmem;
 {void free();
  char* ptrmem;
  ptrmem = (char *)locmem;
  free(ptrmem); }

void laddrs_(arg,addrarg)
long long *arg, **addrarg;
   { *addrarg = arg ; }

void naptime_(long *nptime)
  { long istat;
    istat=(long) sleep((unsigned)*nptime); }

#endif

/*-------------- Silicon Graphics, Incorporated --------------*/

#ifdef SGI32
#define FORTINT int
#endif
#ifdef SGI64
#define FORTINT long
#endif

#if (defined SGI32) || (defined SGI64)

int memget_(nwords) FORTINT *nwords;
   { FORTINT nbytes;
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) FORTINT *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime_(FORTINT *nptime)
   { sginap((long) *nptime); }

#endif

/*----------------- SUN -----------------*/

#ifdef SUN32
#define FORTINT int
#endif
#ifdef SUN64
#define FORTINT long
#endif

#if (defined SUN32) || (defined SUN64)

FORTINT memget_(nwords) FORTINT *nwords;
   { FORTINT nbytes; FORTINT malloc();
     nbytes = (*nwords+2)*8;
     return malloc(nbytes); }

void memrel_(locmem) FORTINT *locmem;
   { free(*locmem); }

#include <unistd.h>
void naptime_(nptime) FORTINT *nptime;
   { unsigned int istat,delay;
     delay = (unsigned int) *nptime;
     istat = sleep(delay); }

#endif
